%
% function [A00, P, wz, RoCz, zCurrent] = modeLoss( Rmir, Rin, win, z, n, L, options, err, k )
%
% This uses corrFactor, Eout = HG00 * corrFactor
% HG00 amplitude = int dxdy conj(HG00) * HG * Corr = 2/pi/w(z)^2 int dxdy Exp(-2*r^2/w^2) * Corr
% int dxdy exp(-2 r^2/w^2) f(r) = 2 pi int du(0~1) f(r=w*sqrt(-log(u)/2) )
% P = int dxdy Eout^2. This should be one, but due to field calculation 
% or numerical reason, it is slightly off from 1.
% A00 is corrected by dividing by sqrt(norm) so that abs(A00) <= 1
%
% outputs
% A00 : The amplitude of the 00 component in the output field
% P : integral of Eout^2, which should be 1 if all are perfect
% wz : beam size at z
% zCurrent : waist to distance at the mirror location
% 
% inpits
% see help for corrFactor
%    z can ba an array
%

% examples
%{
RoC = 1934; w = 0.053; L = 3994.5;
Csph = modeLoss( RoC, RoC, w, L, 0, 0, 1 );
Cpar = modeLoss( RoC, RoC, w, L, 0, 0, 2 );
1-abs(Csph+1)^2
1-abs(Cpar+1)^2
RoC = 0.023; w = 0.001; L = 0.01;
Csph = modeLoss( RoC, RoC, w, L, 0, 0, 1 );
Cpar = modeLoss( RoC, RoC, w, L, 0, 0, 2 );
1-abs(Csph+1)^2
1-abs(Cpar+1)^2
%}
%
function [A00, P, wz, Ropt, zCurrent] = modeLoss( Rmir, Rin, win, z, n, L, options, err, k )

if ~exist( 'options', 'var' )
    options = 0;
end

if ~exist('n', 'var') || n <= 0
    n = -1;
end

if ~exist( 'k', 'var' )
    k = 2*pi/1.064e-6;
end
if ~exist( 'err', 'var' )
    err = 1e-5;
end

if n > 0 && exist('L', 'var') && L > 0
    [Rin, win] = RWSubs( Rin, win, n, L );
else
    L = 0;
end
Ropt = 1/((n-1)/Rmir + 1/Rin);

% beam size at z
wz = sqrt(k^2 * win^4 * (z+Ropt).^2 + 4*z.^2*Ropt^2) / (k*abs(Ropt)*win);

zCurrent = Ropt * k^2 * win^4 / ( k^2*win^4 + 4*Ropt^2 );

P = zeros(1,length(z));
A00 = zeros(1,length(z));
lowerLimit = min(exp(-2*4^2), err);

for iz = 1 : length(z)
    P(iz) = integral( @(u) abs(corrFactor( Rmir, Rin, win, wz(iz)*sqrt(-log(u)/2), 0, z(iz), n, L, options, err, k )).^2, ...
        lowerLimit, 1, 'RelTol', err );
    A00(iz) = integral( @(u) corrFactor( Rmir, Rin, win, wz(iz)*sqrt(-log(u)/2), 0, z(iz), n, L, options, err, k ), ...
        lowerLimit, 1, 'RelTol', err );
end

return
